use crate::{Context, Handler};
use actix_web::{get, web, HttpResponse, Responder};
use serde::{Deserialize, Serialize};
use serde_json::json;
use verax::Asset;

#[derive(Deserialize)]
pub struct AccountId(verax::AccountId);

#[derive(Serialize)]
pub struct Balance {
    amount: String,
    cents: String,
    asset: Asset,
}

impl From<verax::Amount> for Balance {
    fn from(amount: verax::Amount) -> Self {
        Balance {
            amount: amount.to_string(),
            cents: amount.cents().to_string(),
            asset: amount.asset().clone(),
        }
    }
}

#[async_trait::async_trait]
impl Handler for AccountId {
    type Ok = Vec<Balance>;
    type Err = verax::Error;

    async fn handle(self, ctx: &Context) -> Result<Self::Ok, Self::Err> {
        Ok(ctx
            .ledger
            .get_balance(&self.0)
            .await?
            .into_iter()
            .map(|x| x.into())
            .collect())
    }
}

#[get("/balance/{id}")]
pub async fn handler(info: web::Path<AccountId>, ctx: web::Data<Context>) -> impl Responder {
    match info.0.handle(&ctx).await {
        Ok(balances) => HttpResponse::Ok().json(balances),
        Err(err) => HttpResponse::BadRequest().json(json!({ "text": err.to_string(), "err": err})),
    }
}
